@@ -16,12 +16,36 @@ if not hasattr(spacy, 'nlp'):
 	spacy.getdoc	= lambda snt: ( bs := redis.bs.get(snt), doc := spacy.frombs(bs) if bs else spacy.nlp(snt), redis.bs.setnx(snt, spacy.tobs(doc)) if not bs else None )[1]
 
 from elasticsearch import Elasticsearch,helpers
-es = Elasticsearch([ "http://essaydm.wrask.com:9200" ])  
-rows = lambda query, asdic=False: ( res:=requests.post(f"http://essaydm.wrask.com:9200/_sql",json={"query": query}).json().get('rows',[]), dict(res) if asdic else res)[-1]
+es = Elasticsearch([ "http://127.0.0.1:9200" ])  
+rows = lambda query, asdic=False: ( res:=requests.post(f"http://127.0.0.1:9200/_sql",json={"query": query}).json().get('rows',[]), dict(res) if asdic else res)[-1]
 freq = lambda query: rows(query)[0][0]
-mget = lambda ids, index='essaydm': { ar.get('_id',''): ar.get('_source',{})  for ar in requests.post(f"http://essaydm.wrask.com:9200/{index}/_mget",json={"ids": ids}).json().get('docs',[])}
+mget = lambda ids, index='essaydm': { ar.get('_id',''): ar.get('_source',{})  for ar in requests.post(f"http://127.0.0.1:9200/{index}/_mget",json={"ids": ids}).json().get('docs',[])}
 #mget("inau", ['./1941-Roosevelt.txt-28','./1941-Roosevelt.txt-12'])
 
+from st_aggrid import GridOptionsBuilder, AgGrid, GridUpdateMode, DataReturnMode, JsCode
+def grid_gb(df, pagesize=10): 
+	gb = GridOptionsBuilder.from_dataframe(df)
+	#customize gridOptions  pip install streamlit-aggrid
+	gb.configure_default_column(groupable=True, value=True, enableRowGroup=True, aggFunc='sum', editable=True)
+	gb.configure_side_bar()
+	gb.configure_selection("single")
+	gb.configure_pagination(paginationAutoPageSize=False, paginationPageSize=pagesize)
+	gb.configure_grid_options(domLayout='normal')
+	return gb
+
+def grid_response(df, height=345):
+	gb = grid_gb(df) 
+	return AgGrid(
+		df, 
+		gridOptions=gb.build(),
+		height=height, 
+		width='100%',
+		data_return_mode=DataReturnMode.__members__["FILTERED"], 
+		update_mode=GridUpdateMode.__members__["MODEL_CHANGED"],
+		fit_columns_on_grid_load=True, # auto fit 
+		allow_unsafe_jscode=True, #Set it to True to allow jsfunction to be injected
+		)
+
 from math import log as ln
 def likelihood(a,b,c,d, minus=None):  #from: http://ucrel.lancs.ac.uk/llwizard.html
 	try:
