@@ -166,12 +166,14 @@ class ES(object):
 	def hello(self): print (self.es)
 
 	def load(self, infile, idxname, batch=100000): 
-		''' load id-source-file into index, 2022.3.29 '''
+		''' load id-source-file into index, 2022.4.1 '''
+		print(">>started: " , infile, idxname, flush=True )
 		actions=[]
 		for line in readline(infile): 
 			try:
 				arr = json.loads(line) 
-				arr.update({'_op_type':'index', '_index':idxname,}) 
+				#arr.update({'_op_type':'index', '_index':idxname,}) 
+				actions.append( {'_op_type':'index', '_index':idxname, '_id': arr.get('_id',None), '_source': arr.get('_source',{}) } )
 				if len(actions) > batch: 
 					helpers.bulk(client=self.es,actions=actions, raise_on_error=False)
 					print ( actions[-1], flush=True)
@@ -179,7 +181,7 @@ class ES(object):
 			except Exception as e:
 				print("ex:", e)	
 		if actions : helpers.bulk(client=self.es,actions=actions, raise_on_error=False)
-		return actions 
+		print(">>finished " , infile, idxname )
 
 if __name__ == '__main__':
 	fire.Fire(ES)
